﻿(function () {
    var ns = namespace("Intermission");

    ns.startLine = function startLine(index, time) {
        this.lines[index].startTime = time;
        this.lines[index].duration = 1.0; //0.01 * this.lines[index].text.length;
    }

    ns.anyKey = function anyKey(time) {
        this.continue();
    }

    ns.mouseDown = function mouseDown(time) {
        //this.continue();
    }

    ns.continue = function $continue() {
        if (!this.scrollDone) {
            this.index = this.lines.length;
            for (var i = 0; i < this.lines.length; i++) {
                this.lines[i].opacity = 1.0;
            }
            this.scrollDone = true;
        }
        else {
            this._nextScene = true;
        }
    }

    ns.render = function render(batch, time) {
        if (!this.content) {
            this.index = 0;
            this.content = this.Game.Content;
            this.font = this.Game.Font;
            this.backdrop = this.content.LoadTexture("backdrop.png");
            this.graphicsDevice = this.Game.GraphicsDevice;

            this.white = new HdrColors(new HdrColor(255, 255, 255));
            this.black = new HdrColors(new HdrColor(0, 0, 0, 128).Premultiply());
            this.textSize = 30;
            this.linePadding = 5;
            this.rectPadding = 48;

            this.scale = this.graphicsDevice.BackbufferHeight / 720.0;
            this.textSize *= this.scale;
            this.linePadding *= this.scale;
            this.rectPadding *= this.scale;

            this.textColors = [
                new HdrColor(224, 224, 128), new HdrColor(160, 224, 224),
                new HdrColor(32, 32, 96), new HdrColor(32, 32, 96)
            ];

            this.verticalSize = 0;
            this.maxWidth = 0;
            this.lines = [];
            for (var i = 0; i < this.story.Length; i++) {
                var line = {};
                line.text = this.story[i];
                var size = this.font.Measure(line.text, this.textSize, 1.0, true);
                line.size = { x: size.X, y: size.Y };
                this.verticalSize += line.size.y + this.linePadding;
                this.maxWidth = Math.max(this.maxWidth, line.size.x);
                line.opacity = 0.0;
                this.lines.push(line);
            }

            if (this.lines.length > 0) {
                this.startLine(0, time);
            }
        }

        if (this.index < this.lines.length) {
            var line = this.lines[this.index];
            line.opacity = Calc.Clamp(Calc.Lerp(0.0, 1.0, (time - line.startTime) / line.duration));

            if (line.opacity >= 1.0) {
                this.index++;
                if (this.index < this.lines.length) {
                    this.startLine(this.index, time);
                }
                else {
                    this.scrollDone = true;
                }
            }
        }

        var width = this.graphicsDevice.BackbufferWidth;
        var height = this.graphicsDevice.BackbufferHeight;
        var y = (height - this.verticalSize) / 2;

        batch.Begin();
        batch.Draw(this.backdrop, new RectangleF(0, 0, width, height), this.white);
        batch.DrawRectangle(
            new RectangleF(
                (width - (this.maxWidth + this.rectPadding * 3)) / 2,
                y - this.rectPadding / 2,
                this.maxWidth + this.rectPadding * 3,
                this.verticalSize + this.rectPadding),
            this.black
        );
        batch.End();

        for (var i = 0; i <= Math.min(this.index, this.lines.length - 1) ; i++) {
            var line = this.lines[i];
            var x = (width - line.size.x) / 2;

            if (line.opacity >= 1.0) {
                this.font.Draw(line.text, this.textSize, new Vector2(x, y + line.size.y - this.linePadding / 2),
                    this.textColors[0], this.textColors[1], true, this.textColors[2], this.textColors[3]);
            }
            else {
                this.font.Draw(line.text, this.textSize, new Vector2(x, y + line.size.y - this.linePadding / 2),
                    HdrColor.Multiply(this.textColors[0], line.opacity),
                    HdrColor.Multiply(this.textColors[1], line.opacity), true,
                    HdrColor.Multiply(this.textColors[2], line.opacity),
                    HdrColor.Multiply(this.textColors[3], line.opacity));
            }
            y += line.size.y + this.linePadding;
        }
    }

    return ns;
}());